LevelX = Core.class(Sprite)

function LevelX:init()
	-- bg
	application:setBackgroundColor(0x5a002d)
	-- game sprite
	local gamesprite = Sprite.new()
	local splatlayer = Sprite.new()
	local playerfrontlayer = Sprite.new()
	local playerbacklayer = Sprite.new()
	-- tiled
	local bglevel01 = Bitmap.new(Texture.new("tiled/level01.png"))
	local fglevel01 = Bitmap.new(Texture.new("tiled/level01.png"))
	-- tiny-ecs
	if self.tiny == nil then
		self.tiny = require "classes/tiny-ecs"
	end
	self.tiny.tworld = self.tiny.world()
	-- some enemies (xspritelayer, x, y, dx, dy)
	local nmes = {}
	for i = 1, 32 do -- we create a couple of enemies (1024 running pretty good!)
		nmes[i] = ENme1.new(
			playerfrontlayer,
			math.random(myappwidth*0.25, myappwidth*1.5), math.random(myappheight*0.25, myappheight*0.5),
			math.random(12, 24)*16, myappheight*0.5,
			splatlayer, playerbacklayer)
		self.tiny.tworld:addEntity(nmes[i])
	end
	-- the player (xspritelayer, x, y)
	self.player1 = EPlayer1.new(playerfrontlayer, 12*16, 12*16, splatlayer, playerbacklayer)
	self.tiny.tworld:addEntity(self.player1)
	-- add systems to tiny-ecs
	self.tiny.tworld:add(
--	 	SDebugDraw.new(self.tiny),
		SAnimation.new(self.tiny),
		SDynamicBodies.new(self.tiny, nmes),
		SNme1.new(self.tiny, nmes),
		SAI.new(self.tiny, self.player1),
		SPlayer1Control.new(self.tiny),
		SPlayer1.new(self.tiny),
		SAABBCollision.new(self.tiny, self.player1, nmes),
		SDrawable.new(self.tiny)
--		SDrawable.new(self.tiny, self.player1, nmes)
	)
	-- HUD
	local function map(v, minSrc, maxSrc, minDst, maxDst, clampValue)
		local newV = (v - minSrc) / (maxSrc - minSrc) * (maxDst - minDst) + minDst
		return not clampValue and newV or clamp(newV, minDst >< maxDst, minDst <> maxDst)
	end
	self.tiny.hud = Sprite.new()
	-- hud lives
	self.tiny.hudlives = {}
	local pixellife
	for i = 1, self.player1.currlives do
		pixellife = Pixel.new(0xffff00, 0.8, 24, 24)
		if i == 1 then pixellife:setPosition(24, 24)
		else pixellife:setPosition(24*i+13.5*(i-1), 24)
		end
		self.tiny.hud:addChild(pixellife)
		self.tiny.hudlives[i] = pixellife
	end
	-- hud health
	local hudhealthwidth = map(self.player1.currhealth, 0, self.player1.totalhealth, 0, 100)
	self.tiny.hudhealth = Pixel.new(0x00ff00, 0.7, hudhealthwidth, 14)
	self.tiny.hudhealth:setPosition(24, 55)
	self.tiny.hud:addChild(self.tiny.hudhealth)
	-- params
	fglevel01:setScale(1.5)
	-- position
	bglevel01:setPosition(0, -4*32)
	fglevel01:setPosition(0*32, -0.75*32)
	-- order
	gamesprite:addChild(bglevel01)
	gamesprite:addChild(splatlayer)
	gamesprite:addChild(playerbacklayer)
	gamesprite:addChild(playerfrontlayer)
	gamesprite:addChild(fglevel01)
	self:addChild(self.tiny.hud)
	-- camera: yourScene is a Sprite in which you put all your graphics
	self.camera = GCam.new(gamesprite, 0.5, 0.5) -- [, anchorX, anchorY]) -- anchor by default is (0.5, 0.5)
	self.camera:setAutoSize(true)
	self.camera:setBounds(640/2, 360/2, 1280-640/2, 360/2)
	self.camera:setFollow(self.player1.sprite)
	self:addChild(self.camera)
	-- listeners
	self:addEventListener("enterBegin", self.onTransitionInBegin, self)
	self:addEventListener("enterEnd", self.onTransitionInEnd, self)
	self:addEventListener("exitBegin", self.onTransitionOutBegin, self)
	self:addEventListener("exitEnd", self.onTransitionOutEnd, self)
end

-- game loop
local dt
function LevelX:onEnterFrame(e)
	dt = e.deltaTime
	if not self.player1.ispaused then
		self.camera:update(dt)
		self.tiny.tworld:update(dt) -- tiny world
	end
end

-- event listeners
function LevelX:onTransitionInBegin() self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self) end
function LevelX:onTransitionInEnd() self:myKeysPressed() end
function LevelX:onTransitionOutBegin() self:removeAllListeners() end
function LevelX:onTransitionOutEnd() end

-- app keys handler
function LevelX:myKeysPressed()
	self:addEventListener(Event.KEY_DOWN, function(e)
		-- menu (mobiles and desktops)
		if e.keyCode == KeyCode.BACK or e.keyCode == KeyCode.ESC then
			scenemanager:changeScene("menu", 1, transitions[2], easings[2])
		end
	end)
end
